@@ -122,8 +122,7 @@ from os.path import exists, expanduser
 from copy import copy
 from warnings import warn
 
-from six import integer_types, string_types
-from six.moves import range, configparser as cp, input
+import configparser as cp
 
 from logilab.common.compat import str_encode as _encode
 from logilab.common.deprecation import deprecated
@@ -370,9 +369,9 @@ def format_option_value(optdict, value):
         value = value.pattern
     elif optdict.get('type') == 'yn':
         value = value and 'yes' or 'no'
-    elif isinstance(value, string_types) and value.isspace():
+    elif isinstance(value, str) and value.isspace():
         value = "'%s'" % value
-    elif optdict.get('type') == 'time' and isinstance(value, (float, ) + integer_types):
+    elif optdict.get('type') == 'time' and isinstance(value, (float, int)):
         value = format_time(value)
     elif optdict.get('type') == 'bytes' and hasattr(value, '__int__'):
         value = format_bytes(value)
