@@ -22,7 +22,45 @@ __docformat__ = "restructuredtext en"
 import os
 import sys
 from warnings import warn
-from functools import wraps
+from functools import WRAPPER_ASSIGNMENTS, WRAPPER_UPDATES
+
+
+def lazy_wraps(wrapped):
+    """
+    This is the equivalent of the @wraps decorator of functools except it won't
+    try to grabs attributes of the targeted function on decoration but on access.
+
+    This is needed because of logilab.common.modutils.LazyObject.
+
+    Indeed: if you try to decorate a LazyObject with @wraps, wraps will try to
+    access attributes of LazyObject and this will trigger the attempt to import
+    the module decorated by LazyObject which you don't want to do when you just
+    want to mark this LazyObject has been a deprecated objet that you only
+    wants to trigger if the user try to use it.
+
+    Usage: like @wraps()
+
+    >>> @lazy_wraps(function)
+    >>> def wrapper(*args, **kwargs): ...
+    """
+
+    def update_wrapper_attributes(wrapper):
+        def __getattribute__(self, attribute):
+            if attribute in WRAPPER_ASSIGNMENTS:
+                return getattr(wrapped, attribute)
+
+            return super(self.__class__, self).__getattribute__(attribute)
+
+        wrapper.__getattribute__ = __getattribute__
+
+        for attribute in WRAPPER_UPDATES:
+            getattr(wrapper, attribute).update(getattr(wrapped, attribute, {}))
+
+        wrapper.__wrapped__ = wrapped
+
+        return wrapper
+
+    return update_wrapper_attributes
 
 
 class DeprecationWrapper(object):
@@ -100,7 +138,7 @@ def callable_renamed(old_name, new_function, version=None):
     old_function()
     >>>
     """
-    @wraps(new_function)
+    @lazy_wraps(new_function)
     def wrapped(*args, **kwargs):
         send_warning((
             f"{old_name} has been renamed and is deprecated, uses {new_function.__name__} "
@@ -126,7 +164,7 @@ def argument_removed(old_argument_name, version=None):
     42
     """
     def _wrap(func):
-        @wraps(func)
+        @lazy_wraps(func)
         def check_kwargs(*args, **kwargs):
             if old_argument_name in kwargs:
                 send_warning(f"argument {old_argument_name} of callable {func.__name__} has been "
@@ -147,12 +185,12 @@ def callable_deprecated(reason=None, version=None, stacklevel=2):
     compatible version.
     """
     def decorator(func):
-        message = reason or 'The function "%s" is deprecated'
-        if '%s' in message:
-            message %= func.__name__
-
-        @wraps(func)
+        @lazy_wraps(func)
         def wrapped(*args, **kwargs):
+            message = reason or 'The function "%s" is deprecated'
+            if '%s' in message:
+                message %= func.__name__
+
             send_warning(message, version, stacklevel + 1)
             return func(*args, **kwargs)
         return wrapped
@@ -163,33 +201,21 @@ def callable_deprecated(reason=None, version=None, stacklevel=2):
 deprecated = callable_renamed(old_name="deprecated", new_function=callable_deprecated)
 
 
-def class_deprecated(old_name, parents, class_dict):
-    try:
-        return _class_deprecated(old_name, parents, clsdict)
-    except (NameError, TypeError):
-        # in case of conflicting metaclass situation
-        class DeprecatedClass(*parents):
-            def __init__(self, *args, **kwargs):
-                msg = class_dict.get("__deprecation_warning__", f"{old_name} is deprecated")
-                send_warning(
-                    msg,
-                    stacklevel=class_dict.get("__deprecation_warning_stacklevel__", 3),
-                    version=class_dict.get("__deprecation_warning_version__", None),
-                )
-                super(DeprecatedClass, self).__init__(*args, **kwargs)
+def _generate_class_deprecated():
+    class _class_deprecated(type):
+        """metaclass to print a warning on instantiation of a deprecated class"""
 
-        return DeprecatedClass
+        def __call__(cls, *args, **kwargs):
+            message = getattr(cls, "__deprecation_warning__",
+                              "%(cls)s is deprecated") % {'cls': cls.__name__}
+            send_warning(message, stacklevel=getattr(cls, "__deprecation_warning_stacklevel__", 4),
+                         version=getattr(cls, "__deprecation_warning_version__", None))
+            return type.__call__(cls, *args, **kwargs)
+
+    return _class_deprecated
 
 
-class _class_deprecated(type):
-    """metaclass to print a warning on instantiation of a deprecated class"""
-
-    def __call__(cls, *args, **kwargs):
-        msg = getattr(cls, "__deprecation_warning__",
-                      "%(cls)s is deprecated") % {'cls': cls.__name__}
-        send_warning(msg, stacklevel=getattr(cls, "__deprecation_warning_stacklevel__", 4),
-                     version=getattr(cls, "__deprecation_warning_version__", None))
-        return type.__call__(cls, *args, **kwargs)
+class_deprecated = _generate_class_deprecated()
 
 
 def attribute_renamed(old_name, new_name, version=None):
@@ -250,7 +276,7 @@ def argument_renamed(old_name, new_name, version=None):
     42
     """
     def _wrap(func):
-        @wraps(func)
+        @lazy_wraps(func)
         def check_kwargs(*args, **kwargs):
             if old_name in kwargs and new_name in kwargs:
                 raise ValueError(f"argument {old_name} of callable {func.__name__} has been "
@@ -318,7 +344,21 @@ def class_renamed(old_name, new_class, message=None, version=None):
     class_dict['__deprecation_warning_version__'] = version
     class_dict['__deprecation_warning_stacklevel__'] = 5
 
-    return class_deprecated(old_name, (new_class,), class_dict)
+    try:
+        return class_deprecated(old_name, (new_class,), class_dict)
+    except (NameError, TypeError):
+        # in case of conflicting metaclass situation
+        class DeprecatedClass(new_class):
+            def __init__(self, *args, **kwargs):
+                msg = class_dict.get("__deprecation_warning__", f"{old_name} is deprecated")
+                send_warning(
+                    msg,
+                    stacklevel=class_dict.get("__deprecation_warning_stacklevel__", 3),
+                    version=class_dict.get("__deprecation_warning_version__", None),
+                )
+                super(DeprecatedClass, self).__init__(*args, **kwargs)
+
+        return DeprecatedClass
 
 
 def class_moved(new_class, old_name=None, message=None, version=None):
